"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const path_1 = require("path");
const cmd_1 = require("../../../scripts/cmd");
command_1.core.Messages.importMessagesDirectory(path_1.join(__dirname, '..', '..', '..', '..'));
const messages = command_1.core.Messages.loadMessages('sfdx-dpro-plugin', 'build');
class Build extends command_1.SfdxCommand {
    async run() {
        const directory = this.flags.directory || path_1.join(process.cwd(), 'frontend-apps');
        const projectsDirectoryExist = await this.isDirectory(directory);
        if (!projectsDirectoryExist) {
            this.error(`Apps folder is not exists: ${directory}`);
        }
        const projects = (this.flags.projects && this.flags.projects.split(',')) || await command_1.core.fs.readdir(directory);
        const output = {
            successDetails: [],
            failDetails: []
        };
        for (const project of projects) {
            const projectPath = path_1.join(directory, project);
            const isProject = await this.isProject(projectPath);
            if (!isProject) {
                continue;
            }
            const result = await cmd_1.sh({
                cmd: 'npm',
                args: ['run', 'build'],
                cwd: projectPath
            });
            if (result.status === 0) {
                this.ux.log(`Project ${project} build successfully!`);
                output.successDetails.push({
                    success: true,
                    project,
                    stack: result.stdout
                });
            }
            else {
                // TODO: figure out how to create more descriptive output
                this.ux.error(`Project ${project} build unsuccessfully!`);
                output.failDetails.push({
                    success: false,
                    project,
                    stack: result.stderr
                });
            }
        }
        if (output.failDetails.length > 0) {
            this.error(`Build was failed for next projects: ${output.failDetails.map(item => item.project).join(',')}\
      !\nStack:${output.failDetails.map(item => item.stack).join('\n###############\n')}`, { exit: 1 });
        }
        this.ux.log('Build was successful!');
        // Return an object to be displayed with --json
        return { details: output.successDetails };
    }
    isProject(projectPath) {
        return Promise.all([this.isDirectory(projectPath), this.isPackageJsonExist(projectPath)])
            .then(([isDir, packageExist]) => isDir && packageExist);
    }
    isDirectory(projectPath) {
        return command_1.core.fs.stat(projectPath)
            .then(stats => stats.isDirectory())
            .catch(err => false);
    }
    isPackageJsonExist(projectPath) {
        return command_1.core.fs.stat(path_1.join(projectPath, 'package.json'))
            .then(stat => stat.isFile())
            .catch(err => false);
    }
}
Build.description = messages.getMessage('commandDescription');
Build.examples = [
    '$ sfdx dpro:projects:build --directory <path to projects folder> --projects <comma separated project folder names>',
    '$ sfdx dpro:projects:build -d <path to projects folder>'
];
Build.flagsConfig = {
    version: command_1.flags.version(),
    directory: command_1.flags.string({ char: 'd', description: messages.getMessage('directoryFlagDescription') }),
    projects: command_1.flags.string({ char: 'f', description: messages.getMessage('projectsFlagDescription') })
};
exports.default = Build;
//# sourceMappingURL=build.js.map